import os
import datetime
import subprocess
import multiprocessing
import sqlite3

CAT = "cat"
MAKE = "make"
PORTSDIR = os.getenv("PORTSDIR")
if not PORTSDIR:
    PORTSDIR = "/usr/ports"

def get_ports(strmap):
    portlist = []
    for kv in strmap.strip().split():
        kpd = kv.split(":") # <whatever>:<portsdir>[:whatever]
        basep, category, port = kpd[1].rsplit(os.path.sep, 2)
        portlist.append(os.path.join(category, port))
    return portlist

def get_run_depends(category, port, pdir):
    cmd = [MAKE, "-C", pdir, "-VRUN_DEPENDS"]
    output = subprocess.check_output(cmd).strip()
    return get_ports(output)

def get_build_depends(category, port, pdir):
    cmd = [MAKE, "-C", pdir, "-VBUILD_DEPENDS"]
    output = subprocess.check_output(cmd).strip()
    return get_ports(output)

def get_lib_depends(category, port, pdir):
    cmd = [MAKE, "-C", pdir, "-VLIB_DEPENDS"]
    output = subprocess.check_output(cmd).strip()
    return get_ports(output)

def get_website(category, port, pdir):
    cmd = [MAKE, "-C", pdir, "www-site"]
    output = subprocess.check_output(cmd).strip()
    return output.strip()

def get_description(category, port, pdir):
    cmd = [CAT, os.path.join(pdir, "pkg-descr")]
    cmd = [

def walk_category(cat):
    excludes = ("Mk", "Tools", "Templates", "Keywords", "distfiles", "packages",
                "CVS", ".svn")
    portlist = {}
    if cat in excludes:
        return {}
    catdir = os.path.join(PORTSDIR, cat)
    if not os.path.isdir(catdir):
        return {}
    for port in os.listdir(catdir):
        if port in ("CVS", ".svn"):
            continue
        pdir = os.path.join(catdir, port)
        if not os.path.isdir(pdir):
            continue
        print("Creating entry for %s/%s" % (cat, port))
        entry = {
            "run_depends": get_run_depends(cat, port, pdir),
            "lib_depends": get_lib_depends(cat, port, pdir),
            "build_depends": get_build_depends(cat, port, pdir),
            "using": 0
            # "website": get_website(cat, port, pdir),
            # "description":
        }
        portlist[os.path.join(cat, port)] = entry
    return portlist

if __name__ == "__main__":
    categories = os.listdir(PORTSDIR)
    pool = multiprocessing.Pool()
    results = pool.map(walk_category, categories)
    allports = {}
    for res in results:
        allports.update(res)
    for name, entry in allports.viewitems():
        for k in ("run_depends", "lib_depends", "build_depends"):
            for dep in entry[k]:
                if dep in allports:
                    allports[dep]["using"] += 1
                else:
                    print("invalid reference '%s' for port '%s'" % (dep, name))

    connection = sqlite3.connect("portsdb-%s.db" % datetime.date.today())
    cursor = connection.cursor()
    with open(os.path.join("..", "data", "portsdb.sql", "r")) as fp:
        sqlcreate = fp.read().strip()
    cursor.execute(sqlcreate)
    cursor.commit()

    for name, entry in allports.viewitems():
        cursor.execute("""
INSERT INTO ports (name, www, description) VALUES ('%s', '', '');
""" % name)

        for dep in entry["run_depends"]:


        rdeps = str(entry["run_depends"]).replace("'", "\"")
        ldeps = str(entry["lib_depends"]).replace("'", "\"")
        bdeps = str(entry["build_depends"]).replace("'", "\"")
        fp.write('"%s": { "run_depends": %s, "lib_depends": %s, "build_depends": %s, "using": %d },\n' % (name, rdeps, ldeps, bdeps , entry["using"]))
        fp.seek(-2, os.SEEK_CUR)
        fp.write('}, "count": %d }}\n' % len(allports))
    print("Done")
